@@ -9,7 +9,7 @@ from reportlab.pdfbase import pdfmetrics
 from reportlab.platypus.tables import Table, TableStyle
 
 from conf import _, FONT_PATH, FONT_BOLD_PATH
-from api import Invoice
+from api import Invoice, QrCodeBuilder
 
 class BaseInvoice(object):
 
@@ -24,10 +24,16 @@ class BaseInvoice(object):
 
 class SimpleInvoice(BaseInvoice):
 
-    def gen(self, filename):
+    def gen(self, filename, generate_qr_code=False):
         self.TOP = 260
         self.LEFT = 20
         self.filename = filename
+        if generate_qr_code:
+            qr_builder = QrCodeBuilder(self.invoice)
+        else:
+            qr_builder = None
+
+        self.qr_builder = qr_builder
 
         pdfmetrics.registerFont(TTFont('DejaVu', FONT_PATH))
         pdfmetrics.registerFont(TTFont('DejaVu-Bold', FONT_BOLD_PATH))
@@ -51,6 +57,8 @@ class SimpleInvoice(BaseInvoice):
 
         self.pdf.showPage()
         self.pdf.save()
+        if self.qr_builder:
+            self.qr_builder.destroy()
 
     #############################################################
     ## Draw methods
@@ -187,9 +195,9 @@ class SimpleInvoice(BaseInvoice):
                 items_are_with_tax = True
                 if len(item.description) > 52: i+=5
                 if float(int(item.count)) == item.count:
-                    self.pdf.drawString((LEFT + 68) * mm, (TOP - i) * mm, '%d %s' % (item.count, item.unit))
+                    self.pdf.drawString((LEFT + 68) * mm, (TOP - i) * mm, '%.2f %s' % (item.count, item.unit))
                 else:
-                    self.pdf.drawString((LEFT + 68) * mm, (TOP - i) * mm, '%.1f %s' % (item.count, item.unit))
+                    self.pdf.drawString((LEFT + 68) * mm, (TOP - i) * mm, '%.2f %s' % (item.count, item.unit))
                 self.pdf.drawString((LEFT + 88) * mm, (TOP - i) * mm, '%.2f,- %s' % (item.price, self.invoice.currency))
                 self.pdf.drawString((LEFT + 115) * mm, (TOP - i) * mm, '%.2f,- %s' % (item.total, self.invoice.currency))
                 self.pdf.drawString((LEFT + 137) * mm, (TOP - i) * mm, '%.0f %%' % item.tax)
@@ -198,9 +206,9 @@ class SimpleInvoice(BaseInvoice):
             else:
                 if len(item.description) > 75: i+=5
                 if float(int(item.count)) == item.count:
-                    self.pdf.drawString((LEFT + 104) * mm, (TOP - i) * mm, '%d %s' % (item.count, item.unit))
+                    self.pdf.drawString((LEFT + 104) * mm, (TOP - i) * mm, '%.2f %s' % (item.count, item.unit))
                 else:
-                    self.pdf.drawString((LEFT + 104) * mm, (TOP - i) * mm, '%.1f %s' % (item.count, item.unit))
+                    self.pdf.drawString((LEFT + 104) * mm, (TOP - i) * mm, '%.2f %s' % (item.count, item.unit))
                 self.pdf.drawString((LEFT + 123) * mm, (TOP - i) * mm, '%.2f,- %s' % (item.price, self.invoice.currency))
                 self.pdf.drawString((LEFT + 150) * mm, (TOP - i) * mm, '%.2f,- %s' % (item.total, self.invoice.currency))
                 i+=5
@@ -266,6 +274,14 @@ class SimpleInvoice(BaseInvoice):
             height = float(im.size[1]) / (float(im.size[0])/200.0)
             self.pdf.drawImage(self.invoice.creator.stamp_filename, (LEFT + 98) * mm, (TOP - i - 72) * mm, 200, height)
 
+        if self.qr_builder:
+            qr_filename = self.qr_builder.filename
+            im = Image.open(qr_filename)
+            height = float(im.size[1]) / (float(im.size[0]) / 200.0)
+            self.pdf.drawImage(qr_filename, LEFT * mm, (TOP - i - 100) * mm,
+                               200, height)
+
+
         path = self.pdf.beginPath()
         path.moveTo((LEFT + 110) * mm, (TOP - i - 70) * mm)
         path.lineTo((LEFT + 164) * mm, (TOP - i - 70) * mm)
